
#ifndef __UPF_CONTEXT_H__
#define __UPF_CONTEXT_H__

/*
    NB: Some (many) checks in this file were taken from the Boost library.
        Relevant copyright notice is bellow:
  
        (C) Copyright Boost.org 2001. Permission to copy, use, modify, sell and
        distribute this software is granted provided this copyright notice appears
        in all copies. This software is provided "as is" without express or implied
        warranty, and with no claim as to its suitability for any purpose.
  
        See http://www.boost.org for most recent version.
 */

#define _UPF_STRINGIZE(x)     _UPF_DO_STRINGIZE(x)
#define _UPF_DO_STRINGIZE(x)  #x

/*****************************************************************************
 *                          Check for the compiler:                          *
 *****************************************************************************/

/* GNU C++ compiler: */
#if defined(__GNUC__)
    #ifdef __GNUC_PATCHLEVEL__
      #define _UPF_COMPILER \
        "GNU C++ " _UPF_STRINGIZE(__GNUC__) "." _UPF_STRINGIZE(__GNUC_MINOR__) "." _UPF_STRINGIZE(__GNUC_PATCHLEVEL__)
    #else
      #define _UPF_COMPILER \
        "GNU C++ " _UPF_STRINGIZE(__GNUC__) "." _UPF_STRINGIZE(__GNUC_MINOR__)
    #endif

/* Intel C++ compiler: */
#elif defined(__INTEL_COMPILER)
    #define _UPF_COMPILER \
        "Intel C++ " _UPF_STRINGIZE(__INTEL_COMPILER)

/* Microsoft Visual C++ compiler: */
#elif defined(_MSC_VER) && !defined(__MWERKS__)
    #define _UPF_COMPILER \
        "Microsoft Visual C++ " _UPF_STRINGIZE(_MSC_VER)

/* Borland C++: */
#elif defined(__BORLANDC__)
    #define _UPF_COMPILER \
        "Borland C++ " _UPF_STRINGIZE(__BORLANDC__)

#endif


/*****************************************************************************
 *                      Check for the standard library:                      *
 *****************************************************************************/

#ifdef __cplusplus
        
#include <utility>

/* STLport: */
#if defined(__SGI_STL_PORT) || defined(_STLPORT_VERSION)
    /* STLport typically sits on top of some other library => MUST be first */
    #define _UPF_STDLIB \
        "STLport " _UPF_STRINGIZE(__SGI_STL_PORT)

/* Rogue Wave library: */
#elif defined(_RWSTD_VER)
    #define _UPF_STDLIB \
        "Rogue Wave " _UPF_STRINGIZE(_RWSTD_VER)

/* Dinkumware Library: */
#elif defined(_CPPLIB_VER)
    #define _UPF_STDLIB \
        "Dinkumware " _UPF_STRINGIZE(_CPPLIB_VER)

/* GNU libstdc++ 3: */
#elif defined(__GLIBCPP__)
    #define _UPF_STDLIB \
        "GNU libstdc++ " _UPF_STRINGIZE(__GLIBCPP__)

#endif

#endif



/*****************************************************************************
 *               Define runtime context and compiler string:                 *
 *****************************************************************************/

/* Define human-readable compiler description (_not_ for context-detection): */
#if !defined(_UPF_COMPILER)
    #define _UPF_COMPILER_STRING ""
#else
    #if !defined(_UPF_STDLIB)
        #define _UPF_COMPILER_STRING _UPF_COMPILER
    #else
        #define _UPF_COMPILER_STRING _UPF_COMPILER " with " _UPF_STDLIB
    #endif
#endif

#if !defined(UPF_DISABLE_CONTEXTS) && defined(_UPF_COMPILER) && defined(_UPF_STDLIB)
    #ifdef UPF_VENDOR
        #define UPF_RUNTIME_CONTEXT \
            "upf " UPF_VERSION_STRING ", " _UPF_COMPILER_STRING " (" UPF_VENDOR ")"
    #else
        #define UPF_RUNTIME_CONTEXT \
            "upf " UPF_VERSION_STRING ", " _UPF_COMPILER_STRING
    #endif
#else
    #define UPF_RUNTIME_CONTEXT     NULL
#endif


#endif /* __UPF_CONTEXT_H__ */
