#ifndef __UPF_CXX_ARGS_H__
#define __UPF_CXX_ARGS_H__


namespace upf
{

class Arguments
{
public:
    Arguments(upf_Arguments *args) : m_data(args) {}
    
    void write(void *data, size_t len)
    {
        upf_middleware->funcs.marchaller_write(m_data, data, len);
    }

    void read(void *data, size_t len)
    {
        upf_middleware->funcs.marchaller_read(m_data, data, len);
    }
    
    void seek0()
    {
        upf_middleware->funcs.marchaller_seek0(m_data);
    }

    upf_Arguments *getMarchalled() const { return m_data; }
    
protected:
    upf_Arguments *m_data;
};

class ArgumentsOwned : public Arguments
{
public:
    ArgumentsOwned() : Arguments(&m_stackData)
        { upf_middleware->funcs.marchaller_init(&m_stackData); }
    
    ~ArgumentsOwned()
        { upf_middleware->funcs.marchaller_destroy(&m_stackData); }
    
private:
    upf_Arguments  m_stackData;
};



inline Arguments& operator<<(Arguments& holder, const InterfacePtr& value)
{
    holder.write((void*)&value, sizeof(value));
    return holder;
}
inline Arguments& operator>>(Arguments& holder, InterfacePtr& value)
{
    holder.read(&value, sizeof(value));
    return holder;
}

inline Arguments& operator<<(Arguments& holder, const IID& value)
{
    holder.write((void*)&value, sizeof(IID));
    return holder;
}

inline Arguments& operator>>(Arguments& holder, IID& value)
{
    holder.read(&value, sizeof(IID));
    return holder;
}

#define _UPF_DECLARE_SIMPLE_ARG_OP(type) \
    inline Arguments& operator<<(Arguments& holder, type value) \
    { \
        holder.write(&value, sizeof(type)); \
        return holder; \
    } \
    inline Arguments& operator>>(Arguments& holder, type& value) \
    { \
        holder.read(&value, sizeof(type)); \
        return holder; \
    }

_UPF_DECLARE_SIMPLE_ARG_OP(short)
_UPF_DECLARE_SIMPLE_ARG_OP(long)
_UPF_DECLARE_SIMPLE_ARG_OP(unsigned short)
_UPF_DECLARE_SIMPLE_ARG_OP(unsigned long)
_UPF_DECLARE_SIMPLE_ARG_OP(float)
_UPF_DECLARE_SIMPLE_ARG_OP(double)
_UPF_DECLARE_SIMPLE_ARG_OP(long double)
_UPF_DECLARE_SIMPLE_ARG_OP(upf_uint8_t)
_UPF_DECLARE_SIMPLE_ARG_OP(char)
#ifdef __GNUC__
_UPF_DECLARE_SIMPLE_ARG_OP(long long)
_UPF_DECLARE_SIMPLE_ARG_OP(unsigned long long)
// FIXME : define upf::LongLong and upf::ULongLong !!!
#endif

#undef _UPF_DECLARE_SIMPLE_ARG_OP


inline Arguments& operator<<(Arguments& holder, bool value)
{
    return holder << (upf_uint8_t)(value ? 1 : 0);
}

inline Arguments& operator>>(Arguments& holder, bool& value)
{
    upf_uint8_t temp;
    holder >> temp;
    value = (temp != 0);
    return holder;
}


// Teplate for storing lists of elements of any type that has operator<< defined
template<typename T>
Arguments& operator<<(Arguments& holder, const ::std::vector<T>& arr)
{
    holder << (unsigned long)arr.size();
    for (typename std::vector<T>::const_iterator i = arr.begin(); i != arr.end(); i++)
        holder << *i;
    return holder;
}

template<typename T>
Arguments& operator>>(Arguments& holder, ::std::vector<T>& arr)
{
    unsigned long size;
    T value;
    holder >> size;
    arr.clear();
    arr.reserve(size);
    for (; size != 0; size--)
    {
        holder >> value;
        arr.push_back(value);
    }
    return holder;
}

inline Arguments& operator<<(Arguments& holder, const ::std::string& value)
{
    unsigned long wr, len = value.length();
    holder << len;
    const char *str = value.c_str();
    while (len > 0)
    {
        wr = len < 64 ? len : 64;
        holder.write((void*)str, wr);
        str += wr;
        len -= wr;
    }
    return holder;
}

inline Arguments& operator>>(Arguments& holder, ::std::string& value)
{
    unsigned long len, rd;
    char str[64];
#if (defined(__GNUC__) && __GNUC__ == 2) || defined(_MSC_VER)
    // gcc 2.95 doesn't have std::string::clear() (required by standard)
    value = ::std::string();
#else
    value.clear();
#endif

    holder >> len;
    while (len > 0)
    {
        rd = len < 64 ? len : 64;
        holder.read(&str, rd);
        value.append(str, rd);
        len -= rd;
    }
    
    return holder;
}


// upf object I/O:
inline Arguments& operator<<(Arguments& holder, ::upf::IObject* obj);
template<class T> inline Arguments& operator>>(Arguments& holder, ::upf::Ptr<T>& obj);



} // end of namespace 'upf'


#endif
