
#ifndef __UPF_CXX_FACTORY_H__
#define __UPF_CXX_FACTORY_H__

#include "cxx_api.h"

namespace upf
{

template<class T> class StdFactory;
template<class T> class ClassRegistrator;

template<class T> class StdFactory : public IClassFactory
{
public:
    Ptr<IObject> create()
    {
        T *obj = new T;
        obj->_upf_initInstance();
        return obj; 
    }

private:
    _UPF_DECLARE_CLASS_NO_FRIENDS(StdFactory)
    friend class ClassRegistrator<T>;
};

_UPF_DO_IMPLEMENT_CLASS(StdFactory<T>, template<class T>)
{
    UPF_INTERFACE(upf::IClassFactory)
}


template<class T> class ClassRegistrator
{
public:
    static void prepare(IWriteableClassInfo *classinfo, const char *cxxClassName)
    {
        // convert C++ style name to dot convention
        // (e.g. "::upf::impl::Manager" to "upf.impl.Manager")
        char *c, *classname = new char[strlen(cxxClassName)+1];
        while (*cxxClassName == ':') cxxClassName++;
        for (c = classname; *cxxClassName; cxxClassName++)
        {
            if (*cxxClassName == ':')
            {
                *(c++) = '.';
                cxxClassName++;
            }
            else
                *(c++) = *cxxClassName;
        }
        *c = '\0';

        classinfo->setName(classname);
        delete[] classname;

        // create factory object:
        upf::StdFactory<T>::_upf_initClass(NULL);
        upf::StdFactory<T> *factory = new upf::StdFactory<T>;
        factory->_upf_initInstance();

        classinfo->setFactory(factory);

        T::_upf_initClass(classinfo);
        classinfo->addProperty("ImplLanguage", "C++ (" _UPF_COMPILER_STRING ")");
    }

    static bool registerClass(const char *cxxClassName)
    {
        Ptr<IWriteableClassInfo> ciw = upf::create<IWriteableClassInfo>();
        prepare(ciw, cxxClassName);
        Ptr<IManager> mng = upf::getManager();
        mng->registerClass(ciw->freeze());
        return true;
    }
};



}

#endif
