#ifndef __UPF_CXX_PROXY_H__
#define __UPF_CXX_PROXY_H__

#include <assert.h>

namespace upf {

template<class T> class _ProxyBase : public T
{
public:
    virtual upf_Object* _upf_getABIObject() { return m_object; }

protected:
    enum
    {
        // NB: keep this in sync with code generate from upf.IObject definition
        //     in core.idl !
        METHOD_INCREF = 0,
        METHOD_DECREF = 1,
        METHOD_QUERYINTERFACE = 2
    };
    
    _ProxyBase() : m_refCnt(0), m_object(NULL) { m_dispatch.function = NULL; }

    void init_proxy(upf_Object *object, const upf_InterfaceData& dispatch) 
    {
        m_object = object;
        m_dispatch = dispatch;
        assert(object != NULL);
        assert(object->queryInterface != NULL);
        assert(dispatch.data != NULL);
        _real_incRef();
    }

    virtual ~_ProxyBase()
    {
        assert(m_object != NULL);
        _real_decRef();
    }

    void _real_incRef()
    {
        ArgumentsOwned _upf_args;
        m_dispatch.function(m_dispatch.data, METHOD_INCREF, _upf_args.getMarchalled());
    }
    
    void _real_decRef()
    {
        ArgumentsOwned _upf_args;
        m_dispatch.function(m_dispatch.data, METHOD_DECREF, _upf_args.getMarchalled());
    }
    
    InterfacePtr _real_queryInterface(const IID& interface_id)
    {
        ArgumentsOwned _upf_args;
        _upf_args << interface_id;
        m_dispatch.function(m_dispatch.data, METHOD_QUERYINTERFACE, _upf_args.getMarchalled());
        ::upf::InterfacePtr _upf_retval;
        _upf_args >> _upf_retval;
        return _upf_retval;
    }

    InterfacePtr _proxy_queryInterface(const IID& iid)
    {
        return _real_queryInterface(iid);
    }

    void _proxy_incRef()
        { m_refCnt++; }

    void _proxy_decRef()
        { if (--m_refCnt == 0) delete this; }

    int                m_refCnt;
    upf_Object        *m_object;
    upf_InterfaceData  m_dispatch;
};

}

#endif
