#ifndef __UPF_VERSION_H__
#define __UPF_VERSION_H__

/** \file version.h
    Version information and checks.
    \note Don't include this file directly, use \c <upf/upf.h> instead.
  */

/// Major version number
#define UPF_VERSION_MAJOR     0
/// Minor version number
#define UPF_VERSION_MINOR     1
/// Release version number
#define UPF_VERSION_RELEASE   5
/// String form of version information
#define UPF_VERSION_STRING    "0.1.5"

/// Current UPF ABI version number
#define UPF_ABI_VERSION       2



/** Check if UPF library version is at least \a major.minor.release.
    \hideinitializer
  */
#define UPF_CHECK_VERSION(major,minor,release) \
    ( \
     (UPF_VERSION_MAJOR > (major)) || \
     (UPF_VERSION_MAJOR == (major) && UPF_VERSION_MINOR > (minor)) || \
     (UPF_VERSION_MAJOR == (major) && UPF_VERSION_MINOR == (minor) && \
      UPF_VERSION_RELEASE >= (release)) \
    )

/** Check if the version of UPF ABI matches installed UPF headers. Plugins with
    different ABI versions won't be loaded by the core.
    \param ver ABI version number    
    \note The version is compared against UPF_ABI_VERSION macro
    \hideinitializer
 */
#define UPF_CHECK_ABI_VERSION(ver) \
    ((ver) == UPF_ABI_VERSION)

#endif
    /* __UPF_VERSION_H__ */
