
#
# This script wraps upf-idl and upf-codegen (or, rather, upfparser.py and codegen.py)
# so that one command is enough to create e.g. C++ headers and no intermidiate .upface
# file is needed.
#

import sys, getopt, tempfile, os, atexit
import idlparser, codegen

def getOptions():
    return idlparser.getOptions() + codegen.getOptions()

def run(argv):
    def usage():
        print "\nUsage:", argv[0], "-Llanguage -o<output> [flags] file.idl"
        print """
    The supported flags are:
      -ofilename      Name of the output file/directory (depends on language)
""" + getOptions()

    try:
        opts, files = getopt.getopt(argv[1:], 
                                   "D:I:U:Y:NW:n:VhvqTo:L:")
    except getopt.GetoptError:
        usage()
        sys.exit(2)

    args_idl = [argv[0]]
    args_codegen = [argv[0]]

    for o, a in opts:
        if o == '-h':
            usage()
            sys.exit()
        if o == '-o':
            args_codegen.append('-o' + a)
        if o == '-L':
            args_codegen.append('-L' + a)
        if o == '-D':
            args_idl.append('-D' + a)
        if o == '-I':
            args_idl.append('-I' + a)
            args_idl.append('-Wbinclude=' + a)
        if o == '-U':
            args_idl.append('-U' + a)
        if o == '-D':
            args_idl.append('-D' + a)
        if o == '-n':
            args_idl.append('-n' + a)
        if o == '-Y':
            args_idl.append('-Y' + a)
        if o == '-W':
            args_idl.append('-W' + a)
        if o == '-N':
            args_idl.append('-N')
        if o == '-T':
            args_idl.append('-T')
        if o == '-V':
            args_idl.append('-V')
        if o == '-q':
            args_idl.append('-q')
        if o == '-v':
            args_idl.append('-v')
            
    def cleanup(temp):
        if temp != None and os.path.isfile(temp):
            os.remove(temp)
    
    temp_file = tempfile.mktemp(".upface")
    atexit.register(cleanup, temp_file)
    
    args_idl.append('-o%s' % temp_file)
    args_idl += files
    args_codegen.append(temp_file)
    
    idlparser.run(args_idl)
    codegen.run(args_codegen)


if __name__ == "__main__":
    run(sys.argv)
