#
# upf-idl compiler
#
#
# Based on a script from omniidl and heavily modified. Original notice:
#
#                           Package   : omniidl
# omniidlrun.py             Created on: 1999/10/29
#			    Author    : Duncan Grisby (dpg1)
#
#    Copyright (C) 1999 AT&T Laboratories Cambridge
#
#  This file is part of omniidl.
#
#  omniidl is free software; you can redistribute it and/or modify it
#  under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
#  02111-1307, USA.
#
# Description:
#   
#   IDL compiler command

import sys

def getOptions():
    return """\
      -Dname[=value]  Define name for preprocessor
      -Uname          Undefine name for preprocessor
      -Idir           Include dir in search path for preprocessor
      -nf             Do not warn about unresolved forward declarations
      -nc             Do not treat identifiers differing only in case as an error
      -N              Do not run preprocessor
      -Ycmd           Set command for the preprocessor
      -Wparg[,arg...] Send args to the preprocessor
      -T              Use a temporary file, not a pipe, for preprocessor output
      -V              Print version info then exit
      -h              Print this usage message and exit
      -q              Be quiet
      -v              Trace compilation stages
"""
      
def run(argv):

    # Find the location where UPF files are installed and add it
    # to PYTHONPATH, so that we can run omniidl:

    import sys, os, os.path

    if sys.platform == "win32":
        upfPrefix = os.path.dirname(argv[0])
        idldir = os.path.join(upfPrefix, '..', 'idl') 
    else:
        upfPrefix = os.getenv('UPF_PREFIX')
        if upfPrefix != None:
            idldir = upfPrefix + '/share/idl'
        else:
            idldir = None



    # Try to import all omniidl modules, to check if we found the directory:
    try:
        import _omniidl
    except ImportError, msg:
        sys.stderr.write("\n\n")
        sys.stderr.write("upf-idl: ERROR!\n\n")
        sys.stderr.write("upf-idl: Could not open IDL compiler module " \
                         "_omniidlmodule.so\n")
        sys.stderr.write("upf-idl: Please make sure it is in directory ")
        sys.stderr.write(pylibdir + "\n")
        sys.stderr.write("upf-idl: (or set the PYTHONPATH environment variable)\n")
        sys.stderr.write("\n")
        sys.stderr.write("upf-idl: (The error was `" + str(msg) + "')\n")
        sys.stderr.write("\n\n")
        sys.stderr.flush()
        sys.exit(1)

    try:
        import omniidl.main
    except ImportError, msg:
        sys.stderr.write("\n\n")
        sys.stderr.write("upf-idl: ERROR!\n\n")
        sys.stderr.write("upf-idl: Could not open Python files for IDL compiler\n")
        sys.stderr.write("upf-idl: Please put them in directory " + \
                         pylibdir + "\n")
        sys.stderr.write("upf-idl: (or set the PYTHONPATH environment variable)\n")
        sys.stderr.write("\n")
        sys.stderr.write("upf-idl: (The error was `" + str(msg) + "')\n")
        sys.stderr.write("\n\n")
        sys.stderr.flush()
        sys.exit(1)



    # Run the IDL compiler:

    import getopt

    def handleCmdLine(argv):
        def usage():
            print "\nUsage:", argv[0], "-o<output.upface> [flags] file.idl"
            print """
    The supported flags are:

      -ofilename      Name of the file to write interface description to
      -E              Run preprocessor only, print on stdout
""" + getOptions()

        try:
            opts, files = getopt.getopt(argv[1:],
                                       "D:I:U:EY:NW:n:VhvqTo:")
        except getopt.GetoptError:
            usage()
            sys.exit(2)

        output = None
        args = ['-k', '-K', '-bbackend_upf']

        for o, a in opts:
            if o == '-h':
                usage()
                sys.exit()
            if o == '-o':
                if output != None:
                    sys.stderr.write('Multiple output files specified!');
                output = a.replace('\\', '/')
            if o == '-D':
                args.append('-D' + a)
            if o == '-I':
                args.append('-I' + a.replace('\\', '/'))
                args.append('-Wbinclude=' + a.replace('\\', '/'))
            if o == '-U':
                args.append('-U' + a)
            if o == '-D':
                args.append('-D' + a)
            if o == '-n':
                args.append('-n' + a)
            if o == '-Y':
                args.append('-Y' + a)
            if o == '-W':
                args.append('-W' + a)
            if o == '-E':
                args.append('-E')
            if o == '-N':
                args.append('-N')
            if o == '-T':
                args.append('-T')
            if o == '-V':
                args.append('-V')
            if o == '-q':
                args.append('-q')
            if o == '-v':
                args.append('-v')

        if output == None: output = '-' # stdout
        args.append('-Wboutput=' + output)

        if sys.platform == "win32":
            args.append('-T')

        if os.path.isfile(output):
            os.remove(output)

        if idldir != None:
            args.append('-I'+idldir)       # do it as last one, so that it can be overriden
            args.append('-Wbinclude='+idldir)

        args += map(lambda x: x.replace('\\','/'), files)

        omniidl.main.main(args)

    handleCmdLine(argv)


if __name__ == "__main__":
    run(sys.argv)
