# -*- python -*-
#
#
#   Copyright (C) 2001 Emir Uner
#
# This file is part of xcom

"""Find included files. Borrowed from cxx frontend"""

from omniidl import idlvisitor
from omniidl import idlast
import re, os.path

# Finds included file list
class FindIncludeFiles(idlvisitor.AstVisitor):

    def __init__(self,):
        self.includes = []

    def add(self, node):
        file = node.file()
        if not(file in self.includes):
            self.includes.append(file)

    def getIncludes(self, ast):
        ast.accept(self)  
        self.includes.remove(self.mainfile)
        return self.includes
    
    def visitAST(self, node):
        self.mainfile = node.file()
        self.add(node)
        for d in node.declarations(): d.accept(self)
                
    def visitModule(self, node):
        self.add(node)
        for d in node.definitions(): d.accept(self)
        
    def visitInterface(self, node): self.add(node)
    def visitForward(self, node): self.add(node)
    def visitConst(self, node): self.add(node)
    def visitTypedef(self, node): self.add(node)
    def visitStruct(self, node): self.add(node)
    def visitException(self, node): self.add(node)
    def visitUnion(self, node): self.add(node)
    def visitEnum(self, node): self.add(node)
    
def findIncludes(ast):
    """
    Find the included idl files.
    """

    iv = FindIncludeFiles()
    return iv.getIncludes(ast)
