
#
# This script wraps upf-idl and upf-codegen (or, rather, upfparser.py and codegen.py)
# so that one command is enough to create e.g. C++ headers and no intermidiate .upface
# file is needed.
#

import sys, getopt, tempfile, os, atexit
import idlparser, codegen
from upfparser.iface import *



def bytecode2cxx(string):
    out = '\n'
    cnt = 0
    for s in string[:-1]:
        out += '%i,' % ord(s)
        cnt += 1
        if cnt == 20:
            out += '\n'
            cnt = 0
    out += '%i' % ord(string[-1])
    return 'static const unsigned char __UPF_typeinf[] = {%s\n};\n' % out


def generateTypeinf(data, outfile, lang):
    ti = ''
    ns = {} # namespace
    for d in data:
        for key in d:
            if key not in ns:
                ns[key] = d[key]

    ti = serializeAllInDict(ns)
    if lang == 'cxx':
        ti = bytecode2cxx(ti)
    outfile.write(ti)


def getOptions():
    return idlparser.getOptions() + codegen.getOptions()

def run(argv):
    def usage():
        print "\nUsage:", argv[0], "-o<output> [-Llanguage] [flags] file1.idl ..."
        print """
    The supported flags are:
      -ofilename      Name of the output file/directory (depends on language)
      -Llanguage      Output language, can be unspecified or "cxx"
""" + getOptions()

    try:
        opts, files = getopt.getopt(argv[1:], 
                                   "D:I:U:Y:NW:n:VhvqTo:L:")
    except getopt.GetoptError:
        usage()
        sys.exit(2)

    args_idl = [argv[0]]
    output = sys.stdout
    lang = None

    for o, a in opts:
        if o == '-h':
            usage()
            sys.exit()
        if o == '-o':
            output = open(a, 'wt')
        if o == '-L':
            lang =  a
        if o == '-D':
            args_idl.append('-D' + a)
        if o == '-I':
            args_idl.append('-I' + a)
            args_idl.append('-Wbinclude=' + a)
        if o == '-U':
            args_idl.append('-U' + a)
        if o == '-D':
            args_idl.append('-D' + a)
        if o == '-n':
            args_idl.append('-n' + a)
        if o == '-Y':
            args_idl.append('-Y' + a)
        if o == '-W':
            args_idl.append('-W' + a)
        if o == '-N':
            args_idl.append('-N')
        if o == '-T':
            args_idl.append('-T')
        if o == '-V':
            args_idl.append('-V')
        if o == '-q':
            args_idl.append('-q')
        if o == '-v':
            args_idl.append('-v')
            
    def cleanup(temp):
        if temp != None and os.path.isfile(temp):
            os.remove(temp)
    
    temp_file = tempfile.mktemp(".upface")
    atexit.register(cleanup, temp_file)
    
    args_idl.append('-o%s' % temp_file)
    
    data = []    
    for file in files:
        args2 = args_idl + [file]
        idlparser.run(args2)
        doc = ParsedFile()
        doc.parseFile(temp_file)
        data.append(doc.allEntities)
        doc = None
        os.remove(temp_file)
    
    generateTypeinf(data, output, lang)
    if output != sys.stdout:
        output.close()


if __name__ == "__main__":
    run(sys.argv)
